#!/usr/bin/env python
import sys
from pplox.scanner import Scanner

def main():
    if len(sys.argv) < 3:
        print("Usage: ./your_program.sh tokenize <filename>", file=sys.stderr)
        exit(1)

    command = sys.argv[1]
    filename = sys.argv[2]

    if command != "tokenize":
        print(f"Unknown command: {command}", file=sys.stderr)
        exit(1)

    with open(filename) as file:
        file_contents = file.read()

    scanner = Scanner(file_contents)
    tokens = scanner.scan_tokens()
    for token in tokens:
        print(token.to_string())

if __name__ == "__main__":
    main()



from pplox.scanner import Scanner

def test_parens():
    scanner = Scanner("(()")
    tokens = scanner.scan_tokens()
    assert len(tokens) == 4
    assert tokens[0].to_string() == "LEFT_PAREN ( null"
    assert tokens[1].to_string() == "LEFT_PAREN ( null"
    assert tokens[2].to_string() == "RIGHT_PAREN ) null"
    assert tokens[3].to_string() == "EOF  null"

def test_braces():
    scanner = Scanner("{{}}")
    tokens = scanner.scan_tokens()
    assert len(tokens) == 5
    assert tokens[0].to_string() == "LEFT_BRACE { null"
    assert tokens[1].to_string() == "LEFT_BRACE { null"
    assert tokens[2].to_string() == "RIGHT_BRACE } null"
    assert tokens[3].to_string() == "RIGHT_BRACE } null"
    assert tokens[4].to_string() == "EOF  null"

def test_single_characters():
    scanner = Scanner("({*.,+*})")
    tokens = scanner.scan_tokens()
    assert len(tokens) == 10
    assert tokens[0].to_string() == "LEFT_PAREN ( null"
    assert tokens[1].to_string() == "LEFT_BRACE { null"
    assert tokens[2].to_string() == "STAR * null"
    assert tokens[3].to_string() == "DOT . null"
    assert tokens[4].to_string() == "COMMA , null"
    assert tokens[5].to_string() == "PLUS + null"
    assert tokens[6].to_string() == "STAR * null"
    assert tokens[7].to_string() == "RIGHT_BRACE } null"
    assert tokens[8].to_string() == "RIGHT_PAREN ) null"
    assert tokens[9].to_string() == "EOF  null"

def test_empty_file():
    scanner = Scanner("")
    tokens = scanner.scan_tokens()
    assert len(tokens) == 1
    assert tokens[0].to_string() == "EOF  null"



from enum import Enum

class TokenType(Enum):
    LEFT_PAREN = "LEFT_PAREN"
    RIGHT_PAREN = "RIGHT_PAREN"
    LEFT_BRACE = "LEFT_BRACE"
    RIGHT_BRACE = "RIGHT_BRACE"
    STAR = "STAR"
    DOT = "DOT"
    COMMA = "COMMA"
    PLUS = "PLUS"
    SEMICOLON = "SEMICOLON"
    SLASH = "SLASH"
    MINUS = "MINUS"
    EOF = "EOF"


class Token:
    def __init__(self, type, lexeme, literal, line):
        self.type = type
        self.lexeme = lexeme
        self.literal = literal
        self.line = line
    def to_string(self):
        if self.literal is None:
            literal = "null"
        else:
            literal = self.literal
        return str(self.type.value) + " " + self.lexeme + " " + literal
         


#!/usr/bin/env python
import sys
import argparse
from .scanner import Scanner

def main():
    parser = argparse.ArgumentParser(
        prog='pplox',
        description='A Lox interpreter',
    )
    parser.add_argument('filename')           
    parser.add_argument('-t', '--tokenize', action='store_true')
    args = parser.parse_args()

    with open(args.filename) as file:
        file_contents = file.read()
    
    if args.tokenize:
        scanner = Scanner(file_contents)
        tokens = scanner.scan_tokens()
        for token in tokens:
            print(token.to_string())
    else:
        print("Interpreter not yet implemented", file=sys.stderr)
        sys.exit(1)

if __name__ == "__main__":
    main()



from .token_type import TokenType
from .token import Token

class Scanner:
    def __init__(self, source):
        self.source = source
        self.tokens = []
        self.start = 0
        self.current = 0
        self.line = 1
    def scan_tokens(self):
        while not self.is_at_end():
            self.start = self.current
            self.scan_token()
        self.tokens.append(Token(TokenType.EOF, "", None, self.line))
        return self.tokens
    def is_at_end(self):
        return self.current >= len(self.source)
    def scan_token(self):
        c = self.advance()
        match c:
            case "(":
                self.add_token(TokenType.LEFT_PAREN)
            case ")":
                self.add_token(TokenType.RIGHT_PAREN)
            case "{":
                self.add_token(TokenType.LEFT_BRACE)
            case "}":
                self.add_token(TokenType.RIGHT_BRACE)
            case "*":
                self.add_token(TokenType.STAR)
            case ".":
                self.add_token(TokenType.DOT)
            case ",":
                self.add_token(TokenType.COMMA)
            case "+":
                self.add_token(TokenType.PLUS)
            case ";":
                self.add_token(TokenType.SEMICOLON)
            case "/":
                self.add_token(TokenType.SLASH)
            case "-":
                self.add_token(TokenType.MINUS)
    def advance(self):
        c = self.source[self.current]
        self.current += 1
        return c
    def add_token(self, type, literal=None):
        text = self.source[self.start:self.current]
        self.tokens.append(Token(type, text, literal, self.line))